@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014 - 2021  StorPool.
+# Copyright (c) 2014 - 2022  StorPool.
 # All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -48,7 +48,7 @@ else:
     import urllib.parse as uquote
 
 
-VERSION = '6.0.0'
+VERSION = '7.2.0'
 
 
 SP_DEV_PATH = '/dev/storpool/'
@@ -82,8 +82,11 @@ ClientId = _API_ARG('clientId', sp.ClientId)
 VolumeName = _API_ARG('volumeName', sp.VolumeNameOrGlobalId)
 SnapshotName = _API_ARG('snapshotName', sp.SnapshotNameOrGlobalId)
 PlacementGroupName = _API_ARG('placementGroupName', sp.PlacementGroupName)
+VagId = _API_ARG('vagName', sp.VagId)
 VolumeTemplateName = _API_ARG('templateName', sp.VolumeTemplateName)
 GlobalVolumeId = _API_ARG('globalVolumeId', sp.GlobalVolumeId)
+BucketName = _API_ARG('bucketName', sp.KvsName)
+KeyName = _API_ARG('keyName', sp.KvsKeyName)
 
 
 class _API_METHOD(object):
@@ -117,7 +120,7 @@ class _API_METHOD(object):
         if json is not None:
             args.append(_API_ARG('json', json))
 
-        ftext = 'def func(self, {args}clusterName=None):\n'.format(
+        ftext = 'def func(self, {args}clusterName=None, returnRawAPIData=False):\n'.format(
             args=''.join(arg.defstr() + ", " for arg in args))
         for arg in args:
             ftext += '    {arg} = _validate_{arg}({arg})\n'.format(arg=arg._name)
@@ -128,6 +131,8 @@ class _API_METHOD(object):
         ftext += '\n'
 
         ftext += '    res = self("{method}", {multiCluster}, query, {json}, clusterName=clusterName)\n'.format(method=method, multiCluster=repr(self.multiCluster), json=None if json is None else 'json')
+        ftext += '    if returnRawAPIData:\n'
+        ftext += '        return res\n'
         ftext += '    try:\n'
         ftext += '        return returns(res)\n'
         ftext += '    except InvalidArgumentError as e:\n'
@@ -156,7 +161,8 @@ class _API_METHOD(object):
             doc += "    Returns: {res}\n".format(res=returns.name)
 
         func.__doc__ = doc
-        func.spDoc = self.spDoc
+        if hasattr(self, "spDoc"):
+            func.spDoc = self.spDoc
 
         return func
 
@@ -179,10 +185,12 @@ class ApiOk(object):
     '''
 
 
-@JsonObject(autoName=sp.maybe(sp.SnapshotName))
+@JsonObject(autoName=sp.maybe(sp.SnapshotName), name=sp.maybe(sp.VolumeNameOrGlobalId), globalId=sp.maybe(sp.GlobalVolumeId))
 class ApiOkVolumeCreate(ApiOk):
     '''
     autoName: The name of the transient snapshot used during the creation of the volume.
+    name: The name of the newly-created volume if one was not provided in the VolumeCreate request's parameters.
+    globalId: The globalId of the new volume
     '''
 
 
@@ -231,7 +239,9 @@ class ApiMeta(type):
         cls.spDoc.add_section(name, desc)
 
     def __setattr__(cls, name, func):
-        cls.spDoc.add_call(func.spDoc)
+        doc = getattr(func, "spDoc", None)
+        if doc is not None:
+            cls.spDoc.add_call(func.spDoc)
 
         func = func.compile()
         func.__name__ = func.func_name = name
@@ -263,7 +273,7 @@ class Api(object):
     spDoc = ApiDoc(
         """StorPool API Reference""",
         """
-        Copyright (c) 2014 - 2021  StorPool. All rights reserved.
+        Copyright (c) 2014 - 2022  StorPool. All rights reserved.
 
         This reference document describes the StorPool API version 19.01 and
         the supported API calls.
@@ -293,9 +303,9 @@ class Api(object):
             self._source = {}
 
     @classmethod
-    def fromConfig(klass, cfg=None, **kwargs):
+    def fromConfig(klass, cfg=None, use_env=True, **kwargs):
         if cfg is None:
-            cfg = SPConfig()
+            cfg = SPConfig(use_env=use_env)
         return klass(host=cfg['SP_API_HTTP_HOST'], port=int(cfg['SP_API_HTTP_PORT']), auth=cfg['SP_AUTH_TOKEN'], **kwargs)
 
     def __call__(self, method, multiCluster, query, json=None, clusterName=None):
@@ -348,6 +358,15 @@ class Api(object):
     def volumeDevLinkWait(self, volumeName, attach, pollTime=200 * msec, maxTime=60 * sec):
         return pathPollWait(SP_DEV_PATH + volumeName, attach, True, pollTime, maxTime)
 
+    def kvsTryUpdate(self, BucketName, json=sp.KeyValueBucketSetDesc):
+        try:
+            self.kvsUpdate(BucketName, json)
+            return True
+        except ApiError as e:
+            if (e.json['error'])['name'] != 'runIfVersionIsMismatch':
+                raise
+            return False
+
 
 Api.spDocSection("General",
     """
@@ -369,7 +388,7 @@ Api.spDocSection("General",
     storpool placementGroup hdd addDisk 1
     ```
 
-    Python programs may use the API by importing the Python StorPool bindings (use 'pypi install storpool' to install them):
+    Python programs may use the API by importing the Python StorPool bindings (use 'pip install storpool' to install them):
 
     ```
     # Use the default StorPool configuration settings
@@ -377,6 +396,11 @@ Api.spDocSection("General",
     >>> from storpool import spapi
     >>> api=spapi.Api.fromConfig()
 
+    # Use the default StorPool configuration settings, but do NOT allow environment variables to override them
+
+    >>> from storpool import spapi
+    >>> api=spapi.Api.fromConfig(use_env=False)
+
     # Use an already-created spconfig.SPConfig object
     >>> api=spapi.Api.fromConfig(cfg=cfg)
 
@@ -416,6 +440,11 @@ Api.spDocSection("General",
     }
     ```
 
+    Any of the methods may be invoked with the additional boolean parameter
+    "returnRawAPIData"; if it has a true value, the method call will not
+    construct a Python object representing the return value, but will return
+    a Python dictionary or list corresponding to the JSON response data instead.
+
     The calls that may be used may be found in the file spapi.py.  As a rule of
     thumb, the name of the call is the name of the HTTP query with the first
     letter in lowercase (as above: "peersList()" for the "PeersList" query).
@@ -527,10 +556,6 @@ Api.diskDescribe = GET('DiskDescribe/{diskId}', DiskId, returns=sp.Disk).doc("De
     """
     List all disks including detailed information about the objects on each disk.
     """)
-Api.diskInfo = GET('DiskGetInfo/{diskId}', DiskId, returns=sp.DiskInfo).doc("Get disk info",
-    """
-    List all disks including information about the volumes stored on each disk.
-    """)
 Api.diskEject = POST('DiskEject/{diskId}', DiskId).doc("Eject a disk",
     """ Stop operations on the given disk even if it is not empty. """)
 Api.diskForget = POST('DiskForget/{diskId}', DiskId).doc("Forget a disk",
@@ -558,7 +583,9 @@ Api.diskRetrim = POST("DiskRetrim/{diskId}", DiskId).doc("Retrim disk", """ """)
 Api.spDocSection("Volumes", """ """)
 Api.volumesList = GET('VolumesList', returns=[sp.VolumeSummary], multiCluster=True).doc("List all volumes",
     """ Return configuration information about all the volumes. """)
-Api.volumesStatus = GET('VolumesGetStatus', returns={either(sp.VolumeName, sp.SnapshotName): sp.VolumeStatus}, multiCluster=True).doc("Get volume and snapshot status",
+Api.volumesStatus = GET('VolumesGetStatus', returns={either(sp.SnapshotNameOrGlobalId, sp.VolumeNameOrGlobalId): either(sp.SnapshotStatus, sp.VolumeStatus)}, multiCluster=True).doc("Get volume and snapshot status and stored size",
+    """ Return the status and stored size of each volume and snapshot. """)
+Api.volumesQuickStatus = GET('VolumesGetStatusQuick', returns={either(sp.SnapshotNameOrGlobalId, sp.VolumeNameOrGlobalId): either(sp.SnapshotStatusQuick, sp.VolumeStatusQuick)}, multiCluster=True).doc("Get volume and snapshot status",
     """ Return the status of each volume and snapshot. """)
 Api.volumesSpace = GET('VolumesSpace', returns=[sp.VolumeSpace], multiCluster=True).doc("List total used space by each volume",
     """
@@ -602,7 +629,7 @@ Api.volumesGroupBackup = POST('VolumesGroupBackup', json=sp.VolumesGroupBackupDe
     """
     """)
 
-Api.volumeMoveToRemote = POST('VolumeMoveToRemote/{volumeName}', VolumeName, json=sp.VolumeMoveToRemoteDesc).doc("Move a volume from local cluster to a remote cluster",
+Api.volumeMoveToRemote = POST('VolumeMoveToRemote/{volumeName}', VolumeName, json=sp.VolumeMoveToRemoteDesc).doc("Move a volume from the local cluster to a remote cluster",
    """
    """)
 
@@ -674,6 +701,11 @@ Api.snapshotCreateGroup = POST('VolumesGroupSnapshot', json=sp.GroupSnapshotsSpe
 Api.snapshotFromRemote = POST('SnapshotFromRemote', json=sp.SnapshotFromRemoteDesc).doc("Copy a snapshot from a remote location",
     """
     """)
+
+Api.snapshotMoveToRemote = POST('SnapshotMoveToRemote/{snapshotName}', SnapshotName, json=sp.SnapshotMoveToRemoteDesc).doc("Move a snapshot from the local cluster to a remote cluster",
+   """
+   """)
+
 Api.snapshotExport = POST('SnapshotExport', json=sp.SnapshotExportDesc).doc("Allow a remote location to access a local snapshot",
     """
     """)
@@ -732,6 +764,12 @@ Api.placementGroupDelete = POST('PlacementGroupDelete/{placementGroupName}', Pla
 Api.faultSetsList = GET('FaultSetsList', returns={sp.FaultSetName: sp.FaultSet}).doc("List all fault sets", """ """)
 
 
+Api.vagList = GET('VagList/{vagName}', VagId, returns=sp.Vag).doc("List the properties of a volume allocation group", """ """)
+Api.volumeUpdateVag = POST("VolumeUpdateVag/{volumeName}", VolumeName, json=sp.VolumeUpdateVagDesc).doc("Update the allocation group of a volume", """ """)
+Api.snapshotUpdateVag = POST("SnapshotUpdateVag/{snapshotName}", SnapshotName, json=sp.SnapshotUpdateVagDesc).doc("Update the allocation group of a snapshot", """ """)
+Api.vagUpdate = POST("VagUpdate", json=sp.VagUpdateDesc).doc("Update the targets of a volume allocation group", """ """)
+
+
 Api.spDocSection("Volume Templates",
     """ Templates are a set of rules used for creating many similar volumes. """)
 Api.volumeTemplatesList = GET('VolumeTemplatesList', returns=[sp.VolumeTemplateDesc]).doc("List all volume templates", """ """)
@@ -771,7 +809,6 @@ Api.volumeBalancerVolumeDisks = GET('VolumeBalancerVolumeDisks/{volumeName}', Vo
 Api.volumeBalancerSnapshotDisks = GET('VolumeBalancerSnapshotDisks/{snapshotName}', SnapshotName, returns={sp.DiskId: sp.DiskTarget}).doc("List per disk rebalancing estimates for a given snapshot", """ """)
 Api.volumeBalancerVolumeDiskSets = GET('VolumeBalancerVolumeDiskSets/{volumeName}', VolumeName, returns=sp.VolumeBalancerVolumeDiskSets).doc("Get the disk sets computed by the balancer for a given volume", """ """)
 Api.volumeBalancerSnapshotDiskSets = GET('VolumeBalancerSnapshotDiskSets/{snapshotName}', SnapshotName, returns=sp.VolumeBalancerVolumeDiskSets).doc("Get the disk sets computed by the balancer for a given snapshot", """ """)
-Api.volumeBalancerGroups = GET('VolumeBalancerGroups', returns=[sp.VolumeBalancerAllocationGroup]).doc("List balancer allocation groups", """ """)
 
 Api.spDocSection("iSCSI", "")
 
@@ -803,3 +840,12 @@ Api.spDocSection("Nodes in maintenance", "")
 Api.maintenanceList = GET("MaintenanceList", returns=sp.MaintenanceNodesList).doc("List the nodes in maintenance", "")
 Api.maintenanceSet = POST("MaintenanceSet", json=sp.MaintenanceSetDesc).doc("Set node in maintenance", "")
 Api.maintenanceComplete = POST("MaintenanceComplete", json=sp.MaintenanceCompleteDesc).doc("Complete node's maintenance.", "")
+
+Api.spDocSection("Key Value Store", "")
+
+Api.kvsBuckets = GET("KV", returns=sp.KeyValueBucketsList).doc("List all created key value buckets", "")
+Api.kvsList = GET("KV/List/{bucketName}", BucketName, returns=sp.KeyValueBucket).doc("List a complete key value bucket", "")
+Api.kvsGet = GET("KV/Get/{bucketName}/{keyName}", BucketName, KeyName, returns=sp.KeyValueBucket).doc("List a specific key-value pair from a specific bucket", "")
+Api.kvsCreate = POST('KV/Create/{bucketName}', BucketName).doc("Create a new bucket", "")
+Api.kvsDelete = POST('KV/Delete/{bucketName}', BucketName).doc("Delete a bucket", "")
+Api.kvsUpdate = POST('KV/Set/{bucketName}', BucketName, json=sp.KeyValueBucketSetDesc).doc("Update a bucket", "")
