@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014 - 2021  StorPool.
+# Copyright (c) 2014 - 2022  StorPool.
 # All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -24,6 +24,10 @@ import confget
 
 
 DEFAULTS = {
+    "SP_ABRTSYNC_ALTERNATIVE_SENDER": "1",
+    "SP_ABRTSYNC_ALTERNATIVE_SENDER_HTTPS": "1",
+    "SP_ABRTSYNC_NEW_REMOTE_ADDRESSES": "reporter.storpool.com",
+    "SP_ABRTSYNC_NEW_REMOTE_PORTS": "443",
     "SP_ABRTSYNC_REMOTE_ADDRESSES":
         "reports.storpool.com,reports1.storpool.com,reports2.storpool.com",
     "SP_ABRTSYNC_REMOTE_PORTS": "2266",
@@ -63,6 +67,16 @@ DEFAULTS = {
         "-g cpuset:storpool.slice/server_5 -g memory:storpool.slice/common",
     "SP_SERVER6_CGROUPS":
         "-g cpuset:storpool.slice/server_6 -g memory:storpool.slice/common",
+    "SP_SERVER7_CGROUPS":
+        "-g cpuset:storpool.slice/server_7 -g memory:storpool.slice/common",
+    "SP_SERVER8_CGROUPS":
+        "-g cpuset:storpool.slice/server_8 -g memory:storpool.slice/common",
+    "SP_SERVER9_CGROUPS":
+        "-g cpuset:storpool.slice/server_9 -g memory:storpool.slice/common",
+    "SP_SERVER10_CGROUPS":
+        "-g cpuset:storpool.slice/server_10 -g memory:storpool.slice/common",
+    "SP_SERVER11_CGROUPS":
+        "-g cpuset:storpool.slice/server_11 -g memory:storpool.slice/common",
     "SP_NVMED_CGROUPS":
         "-g cpuset:storpool.slice/beacon -g memory:storpool.slice/common",
     "SP_CONTROLLER_CGROUPS": "-g cpuset:system.slice -g memory:system.slice",
@@ -88,28 +102,55 @@ DEFAULTS = {
     ),
     "SP_PREFERRED_PORT": "0",
     "SP_NVME_PCI_DRIVER": "storpool_pci",
+    "SP_BD_LOG_OPEN_CLOSE": "1",
 }
 
+ENV_OVERRIDE = ["SP_AUTH_TOKEN", "SP_API_HTTP_HOST", "SP_API_HTTP_PORT"]
+
 
 class SPConfigException(Exception):
     """ An error that occurred during the StorPool configuration parsing. """
 
 
+def get_env_overrides():
+    """Return a dictionary with environment variable overrides."""
+    return dict(
+        item
+        for item in ((name, os.environ.get(name)) for name in ENV_OVERRIDE)
+        if item[1] is not None
+    )
+
+
 class SPConfig(object):
     """ A representation of the StorPool configuration settings.
 
     When constructed, an object of this class will look for the various
     StorPool configuration files, parse them, and store the obtained
     variables and values into its internal dictionary.
-    The object may later be accessed as a dictionary. """
+    The object may later be accessed as a dictionary.
+
+    If the `override_config` parameter is passed to the constructor,
+    the string-to-string dictionary that it contains will be copied and
+    used instead. In that case, no configuration files will be read at all.
+    """
 
     PATH_CONFIG = '/etc/storpool.conf'
     PATH_CONFIG_DIR = '/etc/storpool.conf.d'
 
-    def __init__(self, section=None, missing_ok=False):
-        self._dict = dict()
+    def __init__(
+        self,
+        section=None,
+        missing_ok=False,
+        use_env=True,
+        override_config=None,
+    ):
         self._section = section
-        self.run_confget(missing_ok=missing_ok)
+
+        if override_config is not None:
+            self._dict = dict(override_config)
+        else:
+            self._dict = {}
+            self.run_confget(missing_ok=missing_ok, use_env=use_env)
 
     @classmethod
     def get_config_files(cls, missing_ok=False):
@@ -126,7 +167,7 @@ class SPConfig(object):
             return to_check
         return [path for path in to_check if os.path.isfile(path)]
 
-    def run_confget(self, missing_ok=False):
+    def run_confget(self, missing_ok=False, use_env=True):
         """ Parse the StorPool configuration files by ourselves. """
         if self._section is not None:
             section = self._section
@@ -149,6 +190,9 @@ class SPConfig(object):
             for section in sections:
                 res.update(raw.get(section, {}))
 
+        if use_env:
+            res.update(get_env_overrides())
+
         self._dict = res
 
     def __getitem__(self, key):
