@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014 - 2019  StorPool.
+# Copyright (c) 2014 - 2021  StorPool.
 # All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,6 +18,8 @@
 
 from __future__ import print_function
 
+import io
+
 from . import spapi
 from . import spdoc
 
@@ -27,7 +29,11 @@ def main():
     html = spdoc.Html()
     spapi.Api.spDoc.build(html)
 
-    with open('storpool/ApiDoc.html.template') as tmpl:
+    with io.open(
+        'storpool/ApiDoc.html.template',
+        mode='r',
+        encoding='UTF-8',
+    ) as tmpl:
         for line in tmpl.read().split('\n'):
             if line == '__DOC__':
                 print(html)
