@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014 - 2019  StorPool.
+# Copyright (c) 2014 - 2020, 2022  StorPool.
 # All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -132,4 +132,6 @@ class JsonObjectImpl(object):
     toJson = to_json
 
     _asdict = to_json
-    __str__ = __repr__ = lambda self: str(self.to_json())
+
+    def __str__(self):
+        return str(self.to_json())
