@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014 - 2020  StorPool.
+# Copyright (c) 2014 - 2022  StorPool.
 # All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -139,6 +139,17 @@ def nameValidator(argName, regex, size, *blacklisted):
     return spTypeFun(argName, validator, '''a string({size}), matching {regex}, except {{{blacklisted}}}'''.format(size=size, regex=regex, blacklisted=", ".join(map(str, blacklisted))))
 
 
+def stringSizeValidator(argName, size):
+    def validator(name):
+        if name is None:
+            error('No {argName} specified', argName=argName)
+        if len(name) >= size:
+            error('Invalid {argName}. Max length is {max}', argName=argName, max=size)
+        return name
+
+    return spTypeFun(argName, validator, '''a string with a max length of {max}'''.format(max=size))
+
+
 def volumeSizeValidator(argName):
     def validator(size):
         try:
@@ -173,6 +184,9 @@ REMOTE_CLUSTER_NAME_REGEX = VOLUME_NAME_REGEX
 REMOTE_CLUSTER_NAME_SIZE = 64
 VOLUME_TAG_NAME_REGEX = r'^[A-Za-z0-9_\-.:]+$'
 VOLUME_TAG_VALUE_REGEX = r'^[A-Za-z0-9_\-.:]*$'
+KVS_NAME_REGEX = VOLUME_NAME_REGEX
+KVS_NAME_SIZE = 1023
+KVS_KEY_SIZE = 1023
 
 SECTOR_SIZE = 512
 MAX_CHAIN_LENGTH = 6
@@ -192,6 +206,7 @@ MAX_PEERS_PER_SUBTYPE = 0x1000
 MAX_SERVER_ID = PEER_TYPE_CLIENT - 1
 MAX_BRIDGE_ID = MAX_PEERS_PER_SUBTYPE - 1
 MAX_CLIENT_ID = MAX_PEERS_PER_SUBTYPE - 1
+MAX_CLIENT_ISCSI_ID = MAX_PEERS_PER_SUBTYPE - 1
 MAX_MGMT_ID = MAX_PEERS_PER_SUBTYPE - 1
 
 NO_DISK_ID = 2**32 - 2
@@ -206,6 +221,7 @@ PeerStatus = oneOf('PeerStatus', 'up', 'down')
 ClientStatus = oneOf('ClientStatus', 'running', 'down')
 ServerStatus = oneOf('ServerStatus', 'running', 'waiting', 'booting', 'down')
 BridgeStatus = oneOf('BridgeStatus', 'running', 'joining', 'down')
+iscsiTargetStatus = oneOf('iscsiTargetStatus', 'running', 'joining', 'down')
 ClusterStatus = oneOf('ClusterStatus', 'running', 'waiting', 'down')
 GUID = regex('GUID', r'^0x[0-9a-fA-F]{2,16}$')
 
@@ -216,11 +232,14 @@ ClientId = intRange('ClientID', 1, MAX_CLIENT_ID)
 ServerId = intRange('ServerID', 1, MAX_SERVER_ID)
 MgmtId = intRange('MgmtID', 1, MAX_MGMT_ID)
 BridgeId = intRange('BridgeId', 1, MAX_BRIDGE_ID)
+iscsiTargetId = intRange('iscsiTargetId', 1, MAX_CLIENT_ISCSI_ID)
 
 DiskId = intRange('DiskID', 0, MAX_DISK_ID)
 DiskDescription = regex('DiskDescritpion', DISK_DESC_REGEX)
 DiskIdOrNoDiskId = eitherOr(DiskId, NO_DISK_ID)
 
+TargetsCount = oneOf('TargetsCount', *[2**exp for exp in range(16)])
+
 SnapshotName = nameValidator("SnapshotName", SNAPSHOT_NAME_REGEX, VOLUME_NAME_SIZE, 'list', 'status')
 SnapshotNameOrGlobalId = nameValidator("SnapshotNameOrGlobalId", SNAPSHOT_NAME_OR_GLOBAL_ID_REGEX, VOLUME_NAME_SIZE, 'list', 'status')
 
@@ -235,10 +254,13 @@ VolumeTagValue = nameValidator("VolumeTagValue", VOLUME_TAG_VALUE_REGEX, VOLUME_
 
 PlacementGroupName = nameValidator("PlacementGroupName", PLACEMENT_GROUP_NAME_REGEX, PLACEMENT_GROUP_NAME_SIZE, 'list')
 FaultSetName = PlacementGroupName
+VagId = intRange('VagId', 1, 2**64)
+OverrideId = unlimitedInt('OverrideId', 0, '-')
 VolumeTemplateName = nameValidator("VolumeTemplateName", VOLUME_TEMPLATE_NAME_REGEX, VOLUME_NAME_SIZE, 'list')
 
 Bandwidth = unlimitedInt('Bandwidth', 0, '-')
 IOPS = unlimitedInt('IOPS', 0, '-')
+LimitType = oneOf('LimitType', 'total', 'perGiB')
 AttachmentRights = oneOf('AttachmentRights', 'rw', 'ro')
 
 ObjectState = namedEnum("ObjectState", "OBJECT_UNDEF OBJECT_OK OBJECT_OUTDATED OBJECT_IN_RECOVERY OBJECT_WAITING_FOR_VERSION OBJECT_WAITING_FOR_DISK OBJECT_DATA_NOT_PRESENT OBJECT_DATA_LOST OBJECT_WAINING_FOR_CHAIN OBJECT_WAIT_IDLE".split(' '))
@@ -258,11 +280,15 @@ iSCSIName = r'^[a-z0-9\-.:]+$'
 iSCSIPGName = r'^[A-Za-z0-9_\-.:]+$'
 
 OnAttached = oneOf("OnAttached", "fail", "detach", "detachForce", "export")
-DematerializationStatus = oneOf("DematerializationStatus", "pending", "allObjectsUsed", "complete", "blockedRelocate", "blockedParentRelocate", "blockedParentRemoving", "blockedParentDifferentVag")
-DeleteBlocked = oneOf("DeleteBlocked", "not blocked", "pending", "rebasing", "flushing", "volume not found", "generation mismatch", "disk down", "multiple children", "peer down", "write not completed", "recovering from remote", "parent recovering from remote", "unknown")
+DematerializationStatus = oneOf("DematerializationStatus", "pending", "allObjectsUsed", "complete", "blockedRelocate", "blockedParentRelocate", "blockedParentRemoving", "blockedParentDifferentVag", "destroyed")
+DeleteBlocked = oneOf("DeleteBlocked", "not blocked", "pending", "rebasing", "flushing", "volume not found", "generation mismatch", "disk down", "multiple children", "peer down", "write not completed", "recovering from remote", "parent recovering from remote", "moving to remote", "unknown")
 MultiClusterState = oneOf("MultiClusterState", "owner", "ownerExported", "slaveCopy", "clusterLocal", "clusterLocalExported", "clusterLocalAutoReconcile", "clusterLocalAutoReconcileExported")
 
 
+KvsName = nameValidator("KvsName", KVS_NAME_REGEX, KVS_NAME_SIZE)
+KvsKeyName = stringSizeValidator('KvsKeyName', KVS_KEY_SIZE)
+
+
 # NETWORK
 @JsonObject(mac=MacAddr)
 class NetDesc(object):
@@ -316,7 +342,7 @@ class Server(Service):
     '''
     id: The ID of the service. Currently this is the same as the ID of the node.
     status: down - There is no storpool_server daemon running or it is still recovering its drives from a crashed state. waiting - storpool_server is running but waiting for some disks to appear to prevent split-brain situations. booting - No missing disks; the server is in the process of joining the cluster ...
-    missingDisks: The cluster will remain down until these disks are seen again. This happens in the case of simultaneous failure of the whole cluster (power failure); the servers keep track of where the most recent configuration and data was stored.
+    missingDisks: The cluster will remain down until these disks are seen again. This may happen in case of a simultaneous failure of the whole cluster (e.g. a power failure); the servers keep track of where the most recent configuration and data was stored.
     pendingDisks: Similar to missingDisks, these are the disks that are ready and waiting for the missing ones.
     '''
 
@@ -346,7 +372,15 @@ class Bridge(Service):
     '''
 
 
-@JsonObject(clusterStatus=ClusterStatus, mgmt={MgmtId: Mgmt}, clients={ClientId: Client}, servers={ServerId: Server}, bridges={BridgeId: Bridge})
+@JsonObject(id=iscsiTargetId, status=iscsiTargetStatus)
+class iscsiTarget(Service):
+    '''
+    id: The ID of the service.
+    status: The current status of the iSCSI target service.
+    '''
+
+
+@JsonObject(clusterStatus=ClusterStatus, mgmt={MgmtId: Mgmt}, clients={ClientId: Client}, servers={ServerId: Server}, bridges={BridgeId: Bridge}, iscsiTargets={iscsiTargetId: iscsiTarget})
 class ClusterStatus(object):
     '''
     clusterStatus: The current status of the whole cluster. running - At least one running server; a cluster is formed. waiting - In quorum but negotiations between servers are not over yet. down - No quorum; most likely because more beacons are needed.
@@ -368,7 +402,7 @@ class ClientConfigStatus(object):
 @JsonObject(diskId=DiskId, transactionId=longType, allObjects=int, completedObjects=int, dispatchedObjects=int, unresolvedObjects=internal(int))
 class Task(object):
     '''
-    diskId: The disk ID this task is on.
+    diskId: The ID of the disk this task is on.
     transactionId: An ID associated with the currently running task. This ID is the same for all the tasks running on different disks but initiated by the same action (e.g. when reallocating a volume, all tasks associated with that volume will have the same ID).
     allObjects: The number of all the objects that the task is performing actions on.
     completedObjects: The number of objects that the task has finished working on.
@@ -423,8 +457,8 @@ class DiskSummaryBase(object):
     generationLeft: The last cluster generation when the disk was active on a running server, or -1 if the disk is currently active.
     softEject: The status of the soft-eject process.
     description: A user-defined description of the disk for easier identification of the device.
-    model: The drive's model.
-    serial: The drive's serial.
+    model: The drive's model name.
+    serial: The drive's serial number.
     '''
 
 
@@ -442,7 +476,7 @@ class DownDiskSummary(DiskSummaryBase):
     scrubbingPausedFor=int, scrubbingPaused=bool, lastScrubCompleted=int)
 class UpDiskSummary(DiskSummaryBase):
     '''
-    sectorsCount: The amount of 512-byte sectors on the disk.
+    sectorsCount: The number of 512-byte sectors on the disk.
     noFua: Whether to issue FUA writes to this device.
     noFlush: Whether write-back cache flushing is disabled for this device.
     noTrim: Whether trim-below is disabled for this device.
@@ -459,7 +493,7 @@ class UpDiskSummary(DiskSummaryBase):
     objectsOnDiskSize: Total size occupied by objects. In essence, this is the estimated disk usage by StorPool.
     scrubbingStartedBefore: In seconds.
     scrubbedBytes: For current scrubbing job run.
-    scrubbingBW: Estimate of the disk BS used for scrubbing B/s.
+    scrubbingBW: Estimate of the disk bandwidth used for scrubbing B/s.
     scrubbingFinishAfter: Extimate of when the scrubbing job is expected to finish based on scrubbingBW and current disk usage.
     scrubbingPausedFor: How many seconds has the current scrubbing job been paused.
     scrubbingPaused: Is scrubbing currently paused
@@ -550,8 +584,52 @@ class FaultSet(object):
     '''
 
 
+# VAGs
+@JsonObject(name=VagId, chains=[[DiskId]], diskSetsConstraintsViolated=bool, overrides={OverrideId: [DiskId]},
+    parentVagId=internal(longType), placeAll=PlacementGroupName, placeTail=PlacementGroupName, placeHead=PlacementGroupName,
+    replication=VolumeReplication, reuseServer=maybe(bool), targetDiskSets=[[DiskId]], templateName=eitherOr(VolumeTemplateName, ""),
+    templateId=internal(longType), vagId=internal(longType))
+class Vag(object):
+    '''
+    chains: The list of disks where the chains are stored.
+    diskSetsConstraintsViolated: Whether any constraint violations have been detected for this VAG so that a rebalancing is necessary.
+    overrides: The disk set overrides for this VAG.
+    parentVagId: The ID of the parent volume allocation group.
+    placeAll: The name of a placement group which describes the disks to be used for all but the last replica.
+    placeTail: The name of a placement group which describes the disks to be used for the last replica, the one used for reading.
+    placeHead: The name of a placement group which describes the disks to be used for the first replica.
+    replication: The number of copies/replicas kept.
+    reuseServer: is it allowed to place replicas on the same server
+    targetDiskSets: The sets of disks that the volume's data should be stored on.
+    templateName: The template that the volume/snapshot's settings are taken from.
+    vagId: The ID of this volume allocation group
+    '''
+
+
+@JsonObject(vagId=eitherOr(VagId, "parent"))
+class VolumeUpdateVagDesc:
+    '''
+    vagId: The numeric ID of the volume allocation group or the string "parent".
+    '''
+
+
+@JsonObject(vagId=eitherOr(VagId, "parent"))
+class SnapshotUpdateVagDesc:
+    '''
+    vagId: The numeric ID of the volume allocation group or the string "parent".
+    '''
+
+
+@JsonObject(vagId=str, targetsCount=TargetsCount)
+class VagUpdateDesc:
+    '''
+    vagId: volume allocation group ID
+    targetsCount
+    '''
+
+
 # VOLUME and SNAPSHOT
-@JsonObject(bw=Bandwidth, iops=IOPS)
+@JsonObject(bw=Bandwidth, iops=IOPS, limitType=maybe(LimitType))
 class VolumeLimits(object):
     '''
     bw: Bandwidth limit in KB.
@@ -570,10 +648,10 @@ class VolumeLimits(object):
 class VolumeSummaryBase(VolumeLimits):
     '''
     parentName: The volume/snapshot's parent snapshot.
-    templateName: The template that the volume/snapshot's settings are taken from.
+    templateName: The template that the volume/snapshot's settings are inherited from.
     size: The volume/snapshots's size in bytes.
     replication: The number of copies/replicas kept.
-    globalId: The global identifier.
+    globalId: The globally-unique identifier of the volume or snapshot.
     placeAll: The name of a placement group which describes the disks to be used for all but the last replica.
     placeTail: The name of a placement group which describes the disks to be used for the last replica, the one used for reading.
     placeHead: The name of a placement group which describes the disks to be used for the first replica.
@@ -582,8 +660,8 @@ class VolumeSummaryBase(VolumeLimits):
     objectsCount: The number of objects that the volume/snapshot is comprised of.
     creationTimestamp: The volume's creation timestamp (UNIX timestamp)
     tags: Arbitrary short name/value pairs stored with the volume.
-    clusterName: multicluster calls only - the name of the cluster volume currently recides in
-    clusterId: multicluster calls only - the id of the cluster volume currently recides in
+    clusterName: multicluster calls only - the name of the cluster volume currently resides in
+    clusterId: multicluster calls only - the id of the cluster volume currently resides in
     reuseServer: is it allowed to place replicas on the same server
     '''
 
@@ -603,6 +681,7 @@ class VolumeSummary(VolumeSummaryBase):
     createdFromGlobalId=maybe(internal(GlobalVolumeId)),
     dematerializationStatus=maybe(internal(DematerializationStatus)),
     deleteBlocked=maybe(internal(DeleteBlocked)),
+    destroyed=maybe(internal(bool)),
     volumeMoveSource=maybe(internal(bool)),
     mcState=maybe(internal(MultiClusterState)),
     mcStateVal=maybe(internal(int)))
@@ -613,6 +692,7 @@ class SnapshotSummary(VolumeSummaryBase):
     autoName: Is this snapshot anonymous.
     bound: Is this a bound snapshot. Bound snapshots are garbage-collected as soon as they remain childless and are no longer attached.
     deleted: Is this snapshot currently being deleted.
+    destroyed: Is this snapshot currently being destroyed.
     transient: Is this a transient snapshot. Transient snapshots are internally created when cloning a volume. They cannot be attached as they may be garbage-collected at any time.
     targetDeleteDate: Scheduled date for the snapshot to be deleted. Unix timestamp
     recoveringFromRemote: Is this snapshot's data currently being transferred from a remote location.
@@ -656,31 +736,65 @@ class SnapshotInfo(SnapshotSummary):
     pass
 
 
-@JsonObject(name=either(VolumeNameOrGlobalId, SnapshotNameOrGlobalId), size=VolumeSize, replication=VolumeReplication,
-    status=oneOf("VolumeCurentStatus", "up", "up soon", "data lost", "down"), snapshot=bool, migrating=bool, decreasedRedundancy=bool, balancerBlocked=bool,
-    storedSize=int, onDiskSize=int, syncingDataBytes=int, syncingMetaObjects=int, downBytes=int,
-    downDrives=[DiskId], missingDrives=[DiskId], missingTargetDrives=[DiskId], softEjectingDrives=[DiskId],
-    tags=maybe({VolumeTagName: VolumeTagValue}), clusterName=maybe(ClusterName), clusterId=maybe(ClusterId))
-class VolumeStatus(object):
+@JsonObject(status=oneOf("VolumeCurrentStatus", "up", "up soon", "data lost", "down"),
+    snapshot=bool, migrating=bool, decreasedRedundancy=bool,
+    balancerBlocked=bool, syncingDataBytes=int, syncingMetaObjects=int,
+    downBytes=int, objectsCount=int, upSoonChainsCount=int,
+    downDrives=[DiskId], missingDrives=[DiskId],
+    missingTargetDrives=[DiskId], softEjectingDrives=[DiskId])
+class VolumeStatusQuick(VolumeSummary):
     '''
-    name: The volume's name.
-    size: The volume's size in bytes.
-    replication: The number of copies/replicas kept.
     status: up - The volume is operational. up soon - Synchronizing versions of objects after a disk has come back up. data lost - The last copy of some of the data in the volume has been lost. down - Some or all of the objects of the volume are missing and the volume is not in a state to continue serving operations.
     snapshot: True if this response describes a snapshot instead of a volume.
     migrating: True if there are tasks for reallocation of the volume.
     decreasedRedundancy: True if any of the replicas of the volume are missing.
     balancerBlocked: Can this volume be rebalanced, or is rebalancing impossible with the current placement policy due to for example missing or soft-ejecting drives.
-    storedSize: The number of bytes of client data on the volume. This does not take into account the StorPool replication and overhead, thus it is never larger than the volume size.
-    onDiskSize: The actual size that the objects of this volume occupy on the disks.
     syncingDataBytes: The total number of bytes in objects currently being synchronized (degraded objects or objects with not yet known version)
     syncingMetaObjects: The number of objects currently being synchronized (degraded objects or objects with not yet known version)
     downBytes: The number of bytes of the volume that are not accessible at the moment.
     downDrives: The IDs of the drives that are not accessible at the moment but needed by this volume. The volume will be in the 'down' status until all or some of these drives reappear.
     missingDrives: The IDs of the drives that are not accessible at the moment. The volume has all the needed data on the rest of the disks and can continue serving requests but it is in the 'degraded' status.
-    tags: Arbitrary short name/value pairs stored with the volume.
-    clusterName: multicluster call only - the name of the cluster volume currently recides in
-    clusterId: multicluster call only - the id of the cluster volume currently recides in
+    objectsCount: The number of objects that the volume/snapshot is comprised of.
+    upSoonChainsCount: The number of objects that have to be transferred.
+    '''
+
+
+@JsonObject(status=oneOf("VolumeCurrentStatus", "up", "up soon", "data lost", "down"),
+    snapshot=bool, migrating=bool, decreasedRedundancy=bool,
+    balancerBlocked=bool, syncingDataBytes=int, syncingMetaObjects=int,
+    downBytes=int, objectsCount=int, upSoonChainsCount=int,
+    downDrives=[DiskId], missingDrives=[DiskId],
+    missingTargetDrives=[DiskId], softEjectingDrives=[DiskId])
+class SnapshotStatusQuick(SnapshotSummary):
+    '''
+    status: up - The volume is operational. up soon - Synchronizing versions of objects after a disk has come back up. data lost - The last copy of some of the data in the volume has been lost. down - Some or all of the objects of the volume are missing and the volume is not in a state to continue serving operations.
+    snapshot: True if this response describes a snapshot instead of a volume.
+    migrating: True if there are tasks for reallocation of the volume.
+    decreasedRedundancy: True if any of the replicas of the volume are missing.
+    balancerBlocked: Can this volume be rebalanced, or is rebalancing impossible with the current placement policy due to for example missing or soft-ejecting drives.
+    syncingDataBytes: The total number of bytes in objects currently being synchronized (degraded objects or objects with not yet known version)
+    syncingMetaObjects: The number of objects currently being synchronized (degraded objects or objects with not yet known version)
+    downBytes: The number of bytes of the volume that are not accessible at the moment.
+    downDrives: The IDs of the drives that are not accessible at the moment but needed by this volume. The volume will be in the 'down' status until all or some of these drives reappear.
+    missingDrives: The IDs of the drives that are not accessible at the moment. The volume has all the needed data on the rest of the disks and can continue serving requests but it is in the 'degraded' status.
+    objectsCount: The number of objects that the volume/snapshot is comprised of.
+    upSoonChainsCount: The number of objects that have to be transferred.
+    '''
+
+
+@JsonObject(storedSize=int, onDiskSize=int)
+class VolumeStatus(VolumeStatusQuick):
+    '''
+    storedSize: The number of bytes of client data on the volume. This does not take into account the StorPool replication and overhead, thus it is never larger than the volume size.
+    onDiskSize: The actual size that the objects of this volume occupy on the disks.
+    '''
+
+
+@JsonObject(storedSize=int, onDiskSize=int)
+class SnapshotStatus(SnapshotStatusQuick):
+    '''
+    storedSize: The number of bytes of client data on the volume. This does not take into account the StorPool replication and overhead, thus it is never larger than the volume size.
+    onDiskSize: The actual size that the objects of this volume occupy on the disks.
     '''
 
 
@@ -714,7 +828,7 @@ class VolumePolicyDesc(object):
     '''
 
 
-@JsonObject(name=VolumeName, size=maybe(VolumeSize), parent=maybe(SnapshotNameOrGlobalId), template=maybe(VolumeTemplateName), baseOn=maybe(VolumeNameOrGlobalId))
+@JsonObject(name=maybe(VolumeName), size=maybe(VolumeSize), parent=maybe(SnapshotNameOrGlobalId), template=maybe(VolumeTemplateName), baseOn=maybe(VolumeNameOrGlobalId))
 class VolumeCreateDesc(VolumePolicyDesc):
     '''
     name: The name of the volume to be created.
@@ -746,10 +860,11 @@ class VolumeSnapshotDesc(object):
     '''
 
 
-@JsonObject(rename=maybe(VolumeName), bind=maybe(bool), targetDeleteDate=maybe(int), deleteAfter=maybe(int), tags=maybe({VolumeTagName: VolumeTagValue}))
+@JsonObject(rename=maybe(VolumeName), template=maybe(VolumeTemplateName), bind=maybe(bool), targetDeleteDate=maybe(int), deleteAfter=maybe(int), tags=maybe({VolumeTagName: VolumeTagValue}))
 class SnapshotUpdateDesc(VolumePolicyDesc):
     '''
     rename: The new name to be set.
+    template: The new template that the snapshot's settings should be based on.
     bind: When true bind this snapshot, when false - unbind it. If not set or missing - no change.
     targetDeleteDate: set absolute targetDeleteDate, or 0 to disable automatic deleting. Unix timestamp. targetDeleteDate can not be set in the past
     deleteAfter: set targetDeleteDate relative to the current time on the mgmt node. If not 0 this value will be added to the current time as seconds and set as targetDeleteDate. If 0 it will discard previous targetDeleteDate
@@ -815,14 +930,15 @@ class VolumesReassignWaitDesc(object):
     '''
 
 
-@JsonObject(volume=VolumeNameOrGlobalId, snapshot=bool, client=ClientId, rights=AttachmentRights, pos=AttachmentPos, cluster=maybe(RemoteLocationName), clusterId=maybe(ClusterId))
+@JsonObject(volume=VolumeNameOrGlobalId, snapshot=bool, client=ClientId, rights=AttachmentRights, pos=AttachmentPos, globalId=maybe(GlobalVolumeId), cluster=maybe(RemoteLocationName), clusterId=maybe(ClusterId))
 class AttachmentDesc(object):
     '''
-    snapshot: Whether it is a snapshot or a volume.
-    client: The ID of the client on which it is attached.
+    snapshot: Whether this is a snapshot or a volume.
+    client: The ID of the client on which the volume or snapshot is attached.
     volume: The name of the attached volume.
     rights: Whether the volume is attached as read only or read/write; always ro for snapshots.
     pos: The attachment position on the client; used by the StorPool client to form the name of the internal /dev/spN device node.
+    globalId: The globally-unique identifier of the volume or snapshot.
     cluster: The local name of the cluster of this attachement for the multicluster call
     clusterId: The clusterId of the cluster of this attachement for the multicluster call
     '''
@@ -869,6 +985,7 @@ class VolumeTemplateSpaceEst(VolumeTemplateSpaceEstEntry):
 @JsonObject(id=internal(int), name=VolumeTemplateName, placeAll=PlacementGroupName, placeTail=PlacementGroupName, placeHead=PlacementGroupName, replication=eitherOr(VolumeReplication, "-"),
     volumesCount=int, snapshotsCount=int, removingSnapshotsCount=int,
     size=eitherOr(VolumeSize, 0), totalSize=eitherOr(VolumeSize, 0), onDiskSize=longType, storedSize=longType,
+    volumesSize=maybe(eitherOr(VolumeSize, 0)), snapshotsWithoutChildrenSize=maybe(eitherOr(VolumeSize, 0)), snapshotsWithChildrenSize=maybe(eitherOr(VolumeSize, 0)),
     availablePlaceAll=longType, availablePlaceTail=longType, availablePlaceHead=longType, capacityPlaceAll=longType, capacityPlaceTail=longType, capacityPlaceHead=longType,
     stored=VolumeTemplateSpaceEst)
 class VolumeTemplateStatusDesc(object):
@@ -882,6 +999,9 @@ class VolumeTemplateStatusDesc(object):
     snapshotsCount: The number of snapshots based on this template (incl. snapshots currently being deleted).
     removingSnapshotsCount: The number of snapshots based on this template currently being deleted.
     size: The number of bytes of all volumes based on this template, not counting the StorPool replication and checksums overhead.
+    volumesSize: The number of bytes provisioned for all volumes in this template.
+    snapshotsWithoutChildrenSize: The number of bytes provisioned for all snapshots in this template that do not have children.
+    snapshotsWithChildrenSize: The number of bytes provisioned for all snapshots in this template that have children.
     totalSize: The number of bytes of all volumes based on this template, including the StorPool replication overhead.
     storedSize: The number of bytes of client data on all the volumes based on this template. This does not take into account the StorPool replication and overhead, thus it is never larger than the volume size.
     onDiskSize: The actual on-disk number of bytes occupied by all the volumes based on this template.
@@ -915,11 +1035,12 @@ class VolumeTemplateUpdateDesc(VolumePolicyDesc):
 
 
 # VOLUME RELOCATOR and BALANCER
-@JsonObject(status=oneOf("RelocatorStatus", 'on', 'off', 'blocked'), volumesToRelocate=int)
+@JsonObject(status=oneOf("RelocatorStatus", 'on', 'off', 'blocked'), volumesToRelocate=int, recoveringFromRemote=maybe(int))
 class VolumeRelocatorStatus(object):
     '''
     status: 'off' - relocator is currently turned off. 'on' - relocator is on. 'blocked' - relocation is blocked, most likely due to missing drives.
     volumesToRelocate: Number of volumes currently being relocated.
+    recoveringFromRemote: Number of volumes currently being recovered from a remote location.
     '''
 
 
@@ -1002,38 +1123,6 @@ class UpDiskTarget(object):
 DiskTarget = either(UpDiskTarget, DownDiskTarget)
 
 
-@JsonObject(storedSize=int, objectsCount=int)
-class VolumeBalancerSlot(object):
-    '''
-    storedSize: Number of bytes of client data stored on the corresponding disk set.
-    objectsCount: Number of objects on the corresponding disk set.
-    '''
-
-
-@JsonObject(placeAll=PlacementGroupName, placeTail=PlacementGroupName, placeHead=PlacementGroupName, replication=VolumeReplication,
-    feasible=bool, blocked=bool,
-    size=int, storedSize=int, objectsCount=int,
-    root=either(VolumeNameOrGlobalId, SnapshotNameOrGlobalId), volumes=[either(VolumeNameOrGlobalId, SnapshotNameOrGlobalId)],
-    targetDiskSets=[[DiskId]], slots=[VolumeBalancerSlot], reuseServer=maybe(bool))
-class VolumeBalancerAllocationGroup(object):
-    '''
-    root: The name of this group's root volume or snapshot
-    volumes: The names of all volumes and snapshots in this group.
-    size: The total size of all volumes and snapshots in the group.
-    storedSize: The total number of bytes of client data on all volumes and  snapshots in this group.
-    objectsCount: The total number of objects of all volumes and snapshots in the group.
-    replication: The number of copies/replicas kept.
-    placeAll: The name of a placement group which describes the disks to be used for all but the last replica.
-    placeTail: The name of a placement group which describes the disks to be used for the last replica, the one used for reading.
-    placeHead: The name of a placement group which describes the disks to be used for the first replica.
-    feasible: Can new volumes be allocated with the current placement policy and redundancy constraints.
-    blocked: Can this volume be rebalanced, or is rebalancing impossible with the current placement policy due to for example missing or soft-ejecting drives.
-    targetDiskSets: The current sets of disks that the volume's data should be stored on.
-    slots: Statistics about each of the current disk sets.
-    reuseServer: allow placement of replicas on same server
-    '''
-
-
 @JsonObject(remoteLocation=RemoteLocationName, remoteId=GlobalVolumeId, name=maybe(VolumeName), placeAll=maybe(PlacementGroupName), placeTail=maybe(PlacementGroupName), placeHead=maybe(PlacementGroupName), replication=maybe(VolumeReplication), template=maybe(VolumeTemplateName), export=maybe(bool), tags=maybe({VolumeTagName: VolumeTagValue}))
 class SnapshotFromRemoteDesc(object):
     '''
@@ -1096,8 +1185,8 @@ class VolumeBackupDesc(object):
 @JsonObject(remoteId=GlobalVolumeId, snapshotGlobalId=GlobalVolumeId)
 class VolumesGroupBackupSingle(object):
     '''
-    remoteId: the globally unique id of the backup.
-    snapshotGlobalId: the globally unique id of the backup.
+    remoteId: the globally-unique id of the backup.
+    snapshotGlobalId: the globally-unique id of the backup.
     '''
 
 
@@ -1111,7 +1200,7 @@ class VolumesGroupBackupDesc(object):
 
 
 @JsonObject(cluster=maybe(ClusterName), clusterId=maybe(ClusterId), onAttached=maybe(OnAttached))
-class VolumeMoveToRemoteDesc(object):
+class VolumeMoveToRemoteBase(object):
     '''
     cluster: The name of the target cluster, use this or clusterId
     clusterId: The id of the target cluster, use this or cluster
@@ -1119,6 +1208,16 @@ class VolumeMoveToRemoteDesc(object):
     '''
 
 
+@JsonObject()
+class VolumeMoveToRemoteDesc(VolumeMoveToRemoteBase):
+    pass
+
+
+@JsonObject()
+class SnapshotMoveToRemoteDesc(VolumeMoveToRemoteBase):
+    pass
+
+
 @JsonObject(cluster=maybe(ClusterName), clusterId=maybe(ClusterId))
 class VolumeExportDesc(object):
     '''
@@ -1346,10 +1445,10 @@ class GroupSnapshotsSpec(object):
 class GroupSnapshotResult(object):
     '''
     volume: The name of the source volume.
-    volumeGlobalId: The globally unique id of the volume.
+    volumeGlobalId: The globally-unique id of the volume.
     snapshot: The name of the created snapshot.
-    remoteId: The globally unique id of the created snapshot.
-    snapshotGlobalId: The globally unique id of the created snapshot.
+    remoteId: The globally-unique id of the created snapshot.
+    snapshotGlobalId: The globally-unique id of the created snapshot.
     '''
 
 
@@ -1602,7 +1701,7 @@ class iSCSISessionTcp(object):
     stats=maybe(iSCSISessionStats), tasks=maybe(iSCSISessionTasks), tcp=maybe(iSCSISessionTcp))
 class iSCSISessionInfo(object):
     '''
-    satus: ok or other
+    status: ok or other
     controllerId: controller id
     ISID: session ISID
     timeCreated: unix timestamp when session was established
@@ -1794,3 +1893,29 @@ class VolumeRevertDesc(object):
     '''
     toSnapshot: name of the snapshot to revert to
     '''
+
+# KEY VALUE STORE
+
+
+@JsonObject(name=KvsName, version=longType, pairs=maybe({KvsKeyName: str}))
+class KeyValueBucket(object):
+    '''
+    name: name of the bucket
+    version: change version of the bucket, increments with each change
+    tags: arbitrary short key/value pairs stored in the bucket
+    '''
+
+
+@JsonObject(buckets={KvsName: longType})
+class KeyValueBucketsList(object):
+    '''
+    list of created buckets in the cluster
+    '''
+
+
+@JsonObject(runIfVersionIs=maybe(longType), set={KvsKeyName: str})
+class KeyValueBucketSetDesc(object):
+    '''
+    runIfVersionIs: if versions don't match, don't execute the command
+    set: list of key-value pairs to set / update / delete (empty value = deletion)
+    '''
